﻿package com.learningactionscript3.motion {
	
	import flash.geom.Point;

	public class MotionUtils {
		
		public function MotionUtils() {
			trace("MotionUtils contains only static methods. No instantiation required. To use, simply call method this way: MotionUtils.methodName()");
		}
		
		//actual distance between two points
		public static function distance(x1:Number, y1:Number, x2:Number, y2:Number):Number {
			var dX:Number = x1 - x2;
			var dY:Number = y1 - y2;
			return Math.sqrt(dX * dX + dY * dY);
		}
		
		//used to determine which is closer when distance doesn't matter
		//in this case, square root is not taken, which makes this method faster
		public static function simpleDistance(x1:Number, y1:Number, x2:Number, y2:Number):Number {
			var dX:Number = x1 - x2;
			var dY:Number = y1 - y2;
			return (dX * dX + dY * dY);
		}

		//simple easing: zeno's paradox. returns one value so you can ease x and y
		//  separately if desired
		public static function zeno(orig:Number, dest:Number, reducer:Number):Number {
			return (dest - orig) / reducer;
		}
		
		//circular movement. returns a point (both x and y)
		public static function circular(centerX:Number, centerY:Number, radius:Number, angle:Number):Point {
			var radian:Number = deg2rad(angle);
			var pt:Point = new Point();
			pt.x = centerX + radius * Math.cos(radian);
			pt.y = centerY + radius * Math.sin(radian);
			return pt;
		}
		
		//elasticity using Hooke's Law. returns one value so you can bounce x and y
		//  separately if desired
		public static function elastic(orig:Number, dest:Number, springConst:Number, damp:Number, elas:Number):Number {
			elas += -springConst * (orig - dest);
			return elas *= damp;
		}
		
		//point at an object
		public static function pointAt(objX:Number, objY:Number, pointAtX:Number, pointAtY:Number){
			var rotationRadians:Number = Math.atan2(objY - pointAtY, 
													objX - pointAtX);
			return rad2deg(rotationRadians);
		}
		
		//convert degrees to radians
		public static function deg2rad(deg:Number):Number {
			return deg * (Math.PI/180);		
		}
		
		//convert radians to degrees
		public static function rad2deg(rad:Number):Number {
			return rad / (Math.PI/180);		
		}
	}
	
}
