﻿/*
derived from the Flash Professional fl.motion.Color class, for users of 
--Flash Builder
--Flex Builder
--the command-line Flex SDK
--FDT
--FlashDevelop
--and other ActionScript Editors.

IMPORTANT NOTE. This is only a small excerpt from the fl.motion.Color class to support the material discussed in Learning ActionScript 3.0 (O'Reilly) by Rich Shupe and Zevan Rosser. No features other than instantiation, setTint(), and interpolateColor() will work.

Use:

See the ColorUtilsExample.fla in the source archive for Chapter 8 at http://www.LearningActionScript3.com/downloads, or:
If you are using Flash Professional, simply use the fl.motion.Color class as directed in the book.
If you're using another editor, simply substitute this import statement:

	import com.learningactionscript3.color.ColorUtils

for this import statement:

	import fl.motion.Color;
	
And use this instantiation: 

	new ColorUtils();

instead of

	new Color();
  
Or this use of the static interpolateColor() method:

	ColorUtils.interpolateColor(fromColor:uint, toColor:uint, progress:Number);
	
instead of

	Color.interpolateColor(fromColor:uint, toColor:uint, progress:Number);
	
*/

package com.learningactionscript3.color {
	
	import flash.geom.ColorTransform;

	public class ColorUtils extends ColorTransform {
		
		public function ColorUtils(redMultiplier:Number=1.0,
								   greenMultiplier:Number=1.0, 
								   blueMultiplier:Number=1.0, 
								   alphaMultiplier:Number=1.0, 
								   redOffset:Number=0, 
								   greenOffset:Number=0,
								   blueOffset:Number=0,
								   alphaOffset:Number=0) {
			super(redMultiplier, greenMultiplier, blueMultiplier, alphaMultiplier, 
				  redOffset, greenOffset, blueOffset, alphaOffset);
		}
		
		public function setTint(tintColor:uint, tintMultiplier:Number):void {
			this.redMultiplier = this.greenMultiplier = this.blueMultiplier = 1 - tintMultiplier;
			var r:uint = (tintColor >> 16) & 0xFF;
			var g:uint = (tintColor >>  8) & 0xFF;
			var b:uint =  tintColor        & 0xFF;
			this.redOffset   = Math.round(r * tintMultiplier);
			this.greenOffset = Math.round(g * tintMultiplier);
			this.blueOffset  = Math.round(b * tintMultiplier);
		}
		
		public static function interpolateColor(fromColor:uint, toColor:uint, progress:Number):uint {
			var q:Number = 1 - progress;
			
			var fromA:uint = (fromColor >> 24) & 0xFF;
			var fromR:uint = (fromColor >> 16) & 0xFF;
			var fromG:uint = (fromColor >>  8) & 0xFF;
			var fromB:uint =  fromColor        & 0xFF;
	
			var toA:uint = (toColor >> 24) & 0xFF;
			var toR:uint = (toColor >> 16) & 0xFF;
			var toG:uint = (toColor >>  8) & 0xFF;
			var toB:uint =  toColor        & 0xFF;
			
			var resultA:uint = fromA * q + toA * progress;
			var resultR:uint = fromR * q + toR * progress;
			var resultG:uint = fromG * q + toG * progress;
			var resultB:uint = fromB * q + toB * progress;
			
			var resultColor:uint = resultA << 24 | resultR << 16 | resultG << 8 | resultB;
			return resultColor;		
		}

	}
	
}
