﻿/*
this meter can orient horizontally and vertically.
rather than worrying about:
	--a vertical meter should be 20 x 100 and a horizontal meter should be 100 x 20
	--a vertical meter should use scaleY and a horizontal should use scaleX

This meter is just rotated, as in the example on page 315 of the book.
The registration point is the upper left corner for horizontal meters,
    and the lower left corner for vertical meters, as described.
Therefore, the meter is sized using "long" and "short" sides and scaleX is always used.
To avoid scaleX confusion when using vertical meters, the class uses a setter called "meterScale"

Simple usage example:

import com.learningactionscript3.sound.PeakMeter;

var snd:Sound = new Sound();
snd.load(new URLRequest("song.mp3"));

var channel:SoundChannel = new SoundChannel();
channel = snd.play();

//vertical example
var leftMeterV:PeakMeter = new PeakMeter();
leftMeterV.y = 100;
addChild(leftMeterV);

var rightMeterV:PeakMeter = new PeakMeter();
rightMeterV.x = 25;
rightMeterV.y = 100;
addChild(rightMeterV);


//horizontal example
var leftMeterH:PeakMeter = new PeakMeter("horizontal");
leftMeterH.x = 50;
addChild(leftMeterH);

var rightMeterH:PeakMeter = new PeakMeter("horizontal");
rightMeterH.x = 50;
rightMeterH.y = 25;
addChild(rightMeterH);


addEventListener(Event.ENTER_FRAME, onLoop, false, 0, true);
function onLoop(evt:Event):void {
	leftMeterV.meterScale = channel.leftPeak;
	rightMeterV.meterScale = channel.leftPeak;
	leftMeterH.meterScale = channel.leftPeak;
	rightMeterH.meterScale = channel.leftPeak;
}

*/

package com.learningactionscript3.sound {
	
	import flash.display.Graphics;
	import flash.display.MovieClip;
	import flash.display.GradientType;
	import flash.geom.Matrix;
	import flash.display.Sprite;
	
	public class PeakMeter extends MovieClip {
		
		private var _w:Number;
		private var _h:Number;
		private var _dir:String;
		private var _meterMask:Sprite;

		public function PeakMeter(dir:String="vertical", longSide:Number=100, shortSide:Number=20) {
			_longSide = longSide;
			_shortSide = shortSide;
			_dir = dir;
			
			var meter:Sprite = createMeter();
			addChild(meter);
			
			_meterMask = createMask();
			addChild(_meterMask)
			
			meter.mask = _meterMask;
			
			if (_dir == "vertical") {
				this.rotation = -90;
			}
		}
		
		private function createMeter():Sprite {
			var matrix:Matrix = new Matrix();
			matrix.createGradientBox(_longSide, _shortSide);
			
			var sp:Sprite = new Sprite();
			var g:Graphics = sp.graphics;
			g.beginGradientFill(GradientType.LINEAR, 
								[0x00FFFF, 0x00FF00, 0xFFFF00, 0xFF6600, 0xFF0000], 
								[1, 1, 1, 1, 1], 
								[40, 80, 120, 210, 255], 
								matrix);
			g.drawRect(0, 0, _longSide, _shortSide);
			g.endFill();
			return sp;
		}
		
		private function createMask():Sprite {
			var sp:Sprite = new Sprite();
			var g:Graphics = sp.graphics;
			g.beginFill(0x00FF00);
			g.drawRect(0, 0, _longSide, _shortSide);
			g.endFill();
			return sp;
		}
		
		private function deg2rad(deg:Number):Number {
			return deg * (Math.PI/180);
		}
		
		public function set meterScale(val:Number):void {
			_meterMask.scaleX = val;
		}

	}
	
}
