﻿/*
These are simple example color effects with preset values for tutorial simplicity.

You will often want to customize the degree to which a filter is applied (how much saturation or how much blur, etc.). If desired, you can modify these methods to accept parameter input. 
*/

package com.learningactionscript3.color {
	
	import flash.filters.ColorMatrixFilter;
	import flash.filters.ConvolutionFilter;
	import flash.geom.ColorTransform;

	public class ColorEffects {
		
		public function ColorEffects() {
			trace("ColorEffects contains only static methods. No instantiation required. To use, simply call method this way: ColorEffects.methodName()");
		}
		
		public static function transformDefault():ColorTransform {
			var noChange:ColorTransform = new ColorTransform();
			noChange.redOffset = 0;
			noChange.greenOffset = 0;
			noChange.blueOffset = 0;
			noChange.alphaOffset = 0;
			noChange.redMultiplier = 1;
			noChange.greenMultiplier = 1;
			noChange.blueMultiplier = 1;
			noChange.alphaMultiplier = 1;
			return noChange;
		}

		public static function saturation():ColorTransform {
			var saturation:ColorTransform = new ColorTransform();
			saturation.redOffset = 0;
			saturation.greenOffset = 0;
			saturation.blueOffset = 0;
			saturation.redMultiplier = 1.3;
			saturation.greenMultiplier = 1.1;
			saturation.blueMultiplier = 1.1;
			return saturation;
		}

		public static function invert():ColorTransform {
			var invert:ColorTransform = new ColorTransform();
			invert.redOffset = 255;
			invert.greenOffset = 255;
			invert.blueOffset = 255;
			invert.redMultiplier = -1;
			invert.greenMultiplier = -1;
			invert.blueMultiplier = -1;
			return invert;
		}
		
		public static function grayscale():ColorMatrixFilter {
			var lumRd:Number = .2127;
			var lumGr:Number = .7152;
			var lumBl:Number = .0722;

			var grayscale:ColorMatrixFilter;
			grayscale = new ColorMatrixFilter([lumRd, lumGr, lumBl, 0, 0,
											   lumRd, lumGr, lumBl, 0, 0,
											   lumRd, lumGr, lumBl, 0, 0,
											       0, 	  0,	 0, 1, 0]);
			return grayscale;
		}
		
		public static function black():ConvolutionFilter {
			var blackArr:Array = [0, 0, 0,
								  0, 0, 0,
								  0, 0, 0];
			var conv:ConvolutionFilter = new ConvolutionFilter(3, 3, blackArr);
			return conv;
		}

		public static function convolutionDefault():ConvolutionFilter {
			var noChangeArr:Array = [0, 0, 0,
									 0, 1, 0,
									 0, 0, 0];
			var conv:ConvolutionFilter = new ConvolutionFilter(3, 3, noChangeArr);
			return conv;
		}

		public static function brightness():ConvolutionFilter {
			var brightnessArr:Array = [0, 0, 0,
									   0, 2, 0,
									   0, 0, 0];
			var conv:ConvolutionFilter = new ConvolutionFilter(3, 3, brightnessArr);
			return conv;
		}

		public static function emboss():ConvolutionFilter {
			var embossUpArr:Array = [-1, -1, 0,
								     -1,  1, 1,
								      0,  1, 1];
			var conv:ConvolutionFilter = new ConvolutionFilter(3, 3, embossUpArr);
			return conv;
		}

		public static function reverseEmboss():ConvolutionFilter {
			var embossDownArr:Array = [1,  1,  0,
									   1,  1, -1,
									   0, -1, -1];
			var conv:ConvolutionFilter = new ConvolutionFilter(3, 3, embossDownArr);
			return conv;
		}

		public static function blur():ConvolutionFilter {
			var blurOnly:Array = [0, 1, 0,
								  1, 1, 1,
								  0, 1, 0];
			var conv:ConvolutionFilter = new ConvolutionFilter(3, 3, blurOnly, 5);
			return conv;
		}

		public static function sharpen():ConvolutionFilter {
			var sharpenArr:Array = [ 0, -1,  0,
								 -1,  5, -1,
								  0, -1,  0];
			var conv:ConvolutionFilter = new ConvolutionFilter(3, 3, sharpenArr);
			return conv;
		}

		public static function findEdges():ConvolutionFilter {
			var edges:Array = [ 0, -1,  0,
							   -1,  4, -1,
								0, -1,  0];
			var conv:ConvolutionFilter = new ConvolutionFilter(3, 3, edges);
			return conv;
		}

	}
	
}
