﻿package com.learningactionscript3.loading {

    import flash.events.Event;
    import flash.events.EventDispatcher;
    import flash.events.HTTPStatusEvent;
    import flash.events.IOErrorEvent;
    import flash.events.ProgressEvent;
    import flash.events.SecurityErrorEvent;			
    import flash.net.URLLoader;
    import flash.net.URLRequest;
    
	public class LoadURL extends EventDispatcher {
		
		private var _loader:URLLoader;
		private var _loadedData:*;
		private var _path:String;
		private var _verbose:Boolean;
		private var _bytesLoaded:Number = 0;
		private var _bytesTotal:Number = 0;
		private var _loadedPercent:Number = 0;
		
		function LoadURL(path:String, verbose:Boolean=false, 
						 format:String="text") {
			_path = path;
			_verbose = verbose;	
			
			_loader = new URLLoader();
			_loader.dataFormat = format;
			
			addListeners();
			
			try {
            	_loader.load(new URLRequest(path));
            } catch (err:Error) {
                trace("Unable to load url:", err.message);
            }
		}
		
		private function addListeners():void {
            _loader.addEventListener(Event.OPEN, 
									 onOpen, false, 0, true);
            _loader.addEventListener(ProgressEvent.PROGRESS, 
									 onProgress, false, 0, true);
            _loader.addEventListener(HTTPStatusEvent.HTTP_STATUS, 
									 onHTTPStatusEvent, false, 0, true);
            _loader.addEventListener(Event.COMPLETE, 
									 onComplete, false, 0, true);
			_loader.addEventListener(IOErrorEvent.IO_ERROR, 
									 onIOError, false, 0, true);
            _loader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, 
									 onSecurityError, false, 0, true);
		}
		
		private function removeListeners():void {
			_loader.removeEventListener(Event.OPEN, onOpen);
            _loader.removeEventListener(ProgressEvent.PROGRESS, 
										onProgress);
            _loader.removeEventListener(HTTPStatusEvent.HTTP_STATUS, 
										onHTTPStatusEvent);
            _loader.removeEventListener(Event.COMPLETE, onComplete);
			_loader.removeEventListener(IOErrorEvent.IO_ERROR, onIOError);
            _loader.removeEventListener(SecurityErrorEvent.SECURITY_ERROR, 
										onSecurityError);
		}

        private function onProgress(evt:ProgressEvent):void {
			_bytesLoaded = evt.bytesLoaded;
			_bytesTotal = evt.bytesTotal;
			_loadedPercent = _bytesLoaded/_bytesTotal;
			
			if (_verbose) { trace("Loading:", _path, ": progress (percent):", _loadedPercent); }
        }
		
        private function onComplete(evt:Event):void {
            removeListeners();
			
			_loadedData = evt.target.data;
			dispatchEvent(new Event(Event.COMPLETE, true));
			
			if (_verbose) { trace("Loading complete:", _path); }
        }
		
		private function onOpen(evt:Event):void {
            if (_verbose) { trace("Loading begun:", _path); }
        }
		
        private function onHTTPStatusEvent(evt:HTTPStatusEvent):void {
            if (_verbose) { trace("HTTP status code: " + evt.status); }
        }
		
        private function onSecurityError(evt:SecurityErrorEvent):void {
            if (_verbose) { trace("A security error occured:\n", evt.text); }
        }

		private function onIOError(evt:IOErrorEvent):void {
			if (_verbose) { trace("A loading error occurred:\n", evt.text) }
		}
		
		//getters
		public function get loadedBytes():Number {
			return _bytesLoaded;
		}
		
		public function get totalBytes():Number {
			return _bytesTotal;
		}
		
		public function get loadedPercent():String {
			return _loadedPercent
		}
		
		public function get loadedData():* {
			return _loadedData;
		}
		
	}
}